package vip.xiaonuo.modular.people.service.impl;/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;

import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.util.CommonAvatarUtil;
import vip.xiaonuo.modular.people.entity.PeopleList;
import vip.xiaonuo.modular.people.mapper.PeopleListMapper;
import vip.xiaonuo.modular.people.param.PeopleListAddParam;
import vip.xiaonuo.modular.people.param.PeopleListEditParam;
import vip.xiaonuo.modular.people.param.PeopleListIdParam;
import vip.xiaonuo.modular.people.param.PeopleListPageParam;
import vip.xiaonuo.modular.people.service.PeopleListService;

import java.util.List;

/**
 * people_listService接口实现类
 *
 * @author wang
 * @date 2023/12/04 16:16
 **/
@Service
public class PeopleListServiceImpl extends ServiceImpl<PeopleListMapper, PeopleList> implements PeopleListService {

    @Autowired
    private PeopleListMapper peopleListMapper;

    @Override
    public Page<PeopleList> page(PeopleListPageParam peopleListPageParam) {
        QueryWrapper<PeopleList> queryWrapper = new QueryWrapper<>();
        if (ObjectUtil.isNotEmpty(peopleListPageParam.getUserName())) {
            queryWrapper.lambda().eq(PeopleList::getUserName, peopleListPageParam.getUserName());
        }
        if (ObjectUtil.isAllNotEmpty(peopleListPageParam.getSortField(), peopleListPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(peopleListPageParam.getSortOrder());
            queryWrapper.orderBy(true, peopleListPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(peopleListPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(PeopleList::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public CommonResult add(PeopleListAddParam peopleListAddParam) {
        QueryWrapper<PeopleList> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_name", peopleListAddParam.getUserName()).or()
                .eq("nick_name", peopleListAddParam.getNickName()).and(i -> i.eq("org_id", peopleListAddParam.getOrgId()));
        PeopleList one = peopleListMapper.selectOne(queryWrapper);
        if (ObjectUtil.isEmpty(one)) {
            PeopleList peopleList = BeanUtil.toBean(peopleListAddParam, PeopleList.class);
            if(ObjectUtil.isEmpty(peopleList.getImg())) {
                // 设置默认头像
                peopleList.setImg(CommonAvatarUtil.generateImg(peopleList.getUserName()));
            }
            this.save(peopleList);
            return CommonResult.ok();
        } else {
            return new CommonResult(500, "姓名或工号已存在！", false);
        }

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public CommonResult edit(PeopleListEditParam peopleListEditParam) {
        PeopleList peopleList = this.queryEntity(peopleListEditParam.getId());
        if (ObjectUtil.isNotEmpty(peopleListEditParam.getUserName()) || ObjectUtil.isNotEmpty(peopleListEditParam.getNickName())) {
            QueryWrapper<PeopleList> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq(StringUtils.isNotBlank(peopleListEditParam.getUserName()), "user_name", peopleListEditParam.getUserName())
                    .or().eq(StringUtils.isNotBlank(peopleListEditParam.getNickName()), "nick_name", peopleListEditParam.getNickName())
                    .and(i -> i.eq("org_id", peopleList.getOrgId()));

            if (ObjectUtil.isEmpty(peopleListMapper.selectOne(queryWrapper))) {
                BeanUtil.copyProperties(peopleListEditParam, peopleList);
                this.updateById(peopleList);
                return CommonResult.ok();
            } else {
                return new CommonResult(500, "修改的姓名或工号已存在！", false);
            }
        } else {
            BeanUtil.copyProperties(peopleListEditParam, peopleList);
            this.updateById(peopleList);
            return CommonResult.ok();
        }

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<PeopleListIdParam> peopleListIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(peopleListIdParamList, PeopleListIdParam::getId));
    }

    @Override
    public PeopleList detail(PeopleListIdParam peopleListIdParam) {
        return this.queryEntity(peopleListIdParam.getId());
    }

    @Override
    public PeopleList queryEntity(Integer id) {
        PeopleList peopleList = this.getById(id);
        if (ObjectUtil.isEmpty(peopleList)) {
            throw new CommonException("people_list不存在，id值为：{}", id);
        }
        return peopleList;
    }
}
